@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2018 Brazil
-  Copyright(C) 2019 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2018  Brazil
+  Copyright(C) 2019-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 #include "grn_db.h"
 #include "grn_ii.h"
 #include "grn_report.h"
+#include "grn_posting.h"
 
 static grn_rc
 grn_accessor_resolve_one_index_column(grn_ctx *ctx, grn_accessor *accessor,
@@ -75,7 +76,7 @@ grn_accessor_resolve_one_index_column(grn_ctx *ctx, grn_accessor *accessor,
   {
     grn_obj_flags column_value_flags = 0;
     grn_obj column_value;
-    grn_posting add_posting;
+    grn_posting_internal add_posting = {0};
 
     if (column->header.type == GRN_COLUMN_VAR_SIZE) {
       column_value_flags |= GRN_OBJ_VECTOR;
@@ -86,7 +87,7 @@ grn_accessor_resolve_one_index_column(grn_ctx *ctx, grn_accessor *accessor,
 
     add_posting.sid = 0;
     add_posting.pos = 0;
-    add_posting.weight = 0;
+    add_posting.weight_float = 0;
 
     GRN_HASH_EACH_BEGIN(ctx, (grn_hash *)current_res, cursor, id) {
       void *key;
@@ -96,7 +97,7 @@ grn_accessor_resolve_one_index_column(grn_ctx *ctx, grn_accessor *accessor,
       grn_id *tid = key;
       grn_rset_recinfo *recinfo = value;
 
-      add_posting.weight = recinfo->score;
+      add_posting.weight_float = recinfo->score;
 
       GRN_BULK_REWIND(&column_value);
       grn_obj_get_value(ctx, column, *tid, &column_value);
@@ -104,10 +105,10 @@ grn_accessor_resolve_one_index_column(grn_ctx *ctx, grn_accessor *accessor,
       int n_elements = GRN_BULK_VSIZE(&column_value) / sizeof(grn_id);
       for (int i = 0; i < n_elements; i++) {
         add_posting.rid = GRN_RECORD_VALUE_AT(&column_value, i);
-        rc = grn_ii_posting_add(ctx,
-                                &add_posting,
-                                (grn_hash *)*next_res,
-                                GRN_OP_OR);
+        rc = grn_ii_posting_add_float(ctx,
+                                      (grn_posting *)(&add_posting),
+                                      (grn_hash *)*next_res,
+                                      GRN_OP_OR);
         if (rc != GRN_SUCCESS) {
           break;
         }
@@ -148,9 +149,8 @@ grn_accessor_resolve_one_table(grn_ctx *ctx, grn_accessor *accessor,
                    table);
 
   {
-    grn_posting posting;
+    grn_posting_internal posting = {0};
 
-    memset(&posting, 0, sizeof(posting));
     GRN_HASH_EACH_BEGIN(ctx, (grn_hash *)current_res, cursor, id) {
       void *key;
       void *value;
@@ -170,11 +170,11 @@ grn_accessor_resolve_one_table(grn_ctx *ctx, grn_accessor *accessor,
       }
 
       posting.rid = next_record_id;
-      posting.weight = recinfo->score;
-      rc = grn_ii_posting_add(ctx,
-                              &posting,
-                              (grn_hash *)*next_res,
-                              GRN_OP_OR);
+      posting.weight_float = recinfo->score;
+      rc = grn_ii_posting_add_float(ctx,
+                                    (grn_posting *)(&posting),
+                                    (grn_hash *)*next_res,
+                                    GRN_OP_OR);
       if (rc != GRN_SUCCESS) {
         break;
       }
@@ -244,9 +244,12 @@ grn_accessor_resolve_one_data_column_sequential(grn_ctx *ctx,
         break;
       }
       if (found) {
-        grn_posting posting = {0};
+        grn_posting_internal posting = {0};
         posting.rid = id;
-        grn_ii_posting_add(ctx, &posting, (grn_hash *)*next_res, GRN_OP_OR);
+        grn_ii_posting_add_float(ctx,
+                                 (grn_posting *)(&posting),
+                                 (grn_hash *)*next_res,
+                                 GRN_OP_OR);
       }
     } GRN_TABLE_EACH_END(ctx, cursor);
     GRN_OBJ_FIN(ctx, &value);
@@ -261,35 +264,31 @@ grn_accessor_resolve_one_data_column_index(grn_ctx *ctx,
                                            grn_obj *current_res,
                                            grn_obj **next_res)
 {
-  {
-    grn_obj *lexicon;
-    lexicon = grn_ctx_at(ctx, index_datum->index->header.domain);
-    if (grn_obj_id(ctx, lexicon) != current_res->header.domain) {
-      char index_name[GRN_TABLE_MAX_KEY_SIZE];
-      int index_name_size;
-      grn_obj *expected;
-      char expected_name[GRN_TABLE_MAX_KEY_SIZE];
-      int expected_name_size;
+  if (index_datum->index->header.domain != current_res->header.domain) {
+    char index_name[GRN_TABLE_MAX_KEY_SIZE];
+    int index_name_size;
+    grn_obj *expected;
+    char expected_name[GRN_TABLE_MAX_KEY_SIZE];
+    int expected_name_size;
 
-      index_name_size = grn_obj_name(ctx,
-                                     index_datum->index,
-                                     index_name,
-                                     GRN_TABLE_MAX_KEY_SIZE);
-      expected = grn_ctx_at(ctx, current_res->header.domain);
-      expected_name_size = grn_obj_name(ctx,
-                                        expected,
-                                        expected_name,
-                                        GRN_TABLE_MAX_KEY_SIZE);
-      ERR(GRN_INVALID_ARGUMENT,
-          "[accessor][resolve][data-column] lexicon mismatch index: "
-          "<%.*s> "
-          "expected:<%.*s>",
-          index_name_size,
-          index_name,
-          expected_name_size,
-          expected_name);
-      return ctx->rc;
-    }
+    index_name_size = grn_obj_name(ctx,
+                                   index_datum->index,
+                                   index_name,
+                                   GRN_TABLE_MAX_KEY_SIZE);
+    expected = grn_ctx_at(ctx, current_res->header.domain);
+    expected_name_size = grn_obj_name(ctx,
+                                      expected,
+                                      expected_name,
+                                      GRN_TABLE_MAX_KEY_SIZE);
+    ERR(GRN_INVALID_ARGUMENT,
+        "[accessor][resolve][data-column] lexicon mismatch index: "
+        "<%.*s> "
+        "expected:<%.*s>",
+        index_name_size,
+        index_name,
+        expected_name_size,
+        expected_name);
+    return ctx->rc;
   }
 
   grn_report_index(ctx,
@@ -334,18 +333,18 @@ grn_accessor_resolve_one_data_column_index(grn_ctx *ctx,
       }
 
       while ((posting = grn_ii_cursor_next(ctx, ii_cursor))) {
-        grn_posting add_posting;
+        grn_posting_internal add_posting;
 
         if (index_datum->section > 0 && posting->sid != index_datum->section) {
           continue;
         }
 
-        add_posting = *posting;
-        add_posting.weight += recinfo->score;
-        rc = grn_ii_posting_add(ctx,
-                                &add_posting,
-                                (grn_hash *)*next_res,
-                                GRN_OP_OR);
+        add_posting = *((grn_posting_internal *)posting);
+        add_posting.weight_float += recinfo->score;
+        rc = grn_ii_posting_add_float(ctx,
+                                      (grn_posting *)(&add_posting),
+                                      (grn_hash *)*next_res,
+                                      GRN_OP_OR);
         if (rc != GRN_SUCCESS) {
           break;
         }
@@ -387,6 +386,7 @@ grn_accessor_resolve_one_data_column(grn_ctx *ctx, grn_accessor *accessor,
                                                     &index_datum,
                                                     current_res,
                                                     next_res);
+    grn_obj_unref(ctx, index_datum.index);
   }
 
   return rc;
@@ -445,7 +445,7 @@ grn_accessor_resolve(grn_ctx *ctx, grn_obj *accessor, int depth,
       void *value;
       grn_id *record_id;
       grn_rset_recinfo *recinfo;
-      grn_posting posting;
+      grn_posting_internal posting = {0};
 
       grn_hash_cursor_get_key_value(ctx, cursor, &key, NULL, &value);
       record_id = key;
@@ -453,8 +453,11 @@ grn_accessor_resolve(grn_ctx *ctx, grn_obj *accessor, int depth,
       posting.rid = *record_id;
       posting.sid = 1;
       posting.pos = 0;
-      posting.weight = recinfo->score;
-      grn_ii_posting_add(ctx, &posting, (grn_hash *)res, op);
+      posting.weight_float = recinfo->score;
+      grn_ii_posting_add_float(ctx,
+                               (grn_posting *)(&posting),
+                               (grn_hash *)res,
+                               op);
     } GRN_HASH_EACH_END(ctx, cursor);
     grn_obj_unlink(ctx, current_res);
     grn_ii_resolve_sel_and(ctx, (grn_hash *)res, op);
@@ -468,6 +471,51 @@ grn_accessor_resolve(grn_ctx *ctx, grn_obj *accessor, int depth,
   return rc;
 }
 
+grn_id
+grn_accessor_resolve_id(grn_ctx *ctx, grn_obj *accessor, grn_id id)
+{
+  GRN_API_ENTER;
+
+  grn_accessor *a;
+  for (a = (grn_accessor *)accessor;
+       a->next;
+       a = a->next) {
+    switch (a->action) {
+    case GRN_ACCESSOR_GET_KEY :
+      {
+        grn_id next_id;
+        int key_size = grn_table_get_key(ctx,
+                                         a->obj,
+                                         id,
+                                         &next_id,
+                                         sizeof(grn_id));
+        if (key_size != sizeof(grn_id)) {
+          GRN_API_RETURN(id);
+        }
+        id = next_id;
+      }
+      break;
+    case GRN_ACCESSOR_GET_VALUE :
+      {
+        grn_obj value;
+        GRN_VOID_INIT(&value);
+        grn_obj_get_value(ctx, a->obj, id, &value);
+        if (GRN_BULK_VSIZE(&value) < sizeof(grn_id)) {
+          GRN_OBJ_FIN(ctx, &value);
+          GRN_API_RETURN(id);
+        }
+        id = GRN_RECORD_VALUE(&value);
+        GRN_OBJ_FIN(ctx, &value);
+      }
+      break;
+    default :
+      GRN_API_RETURN(id);
+    }
+  }
+
+  GRN_API_RETURN(id);
+}
+
 grn_rc
 grn_accessor_execute(grn_ctx *ctx,
                      grn_obj *accessor,
@@ -884,6 +932,9 @@ grn_accessor_name(grn_ctx *ctx, grn_obj *accessor, grn_obj *name)
         GRN_TEXT_PUT(ctx, name, target_name, target_name_size);
       }
       GRN_TEXT_PUTS(ctx, name, ")");
+      if (show_obj_domain_name && target && grn_enable_reference_count) {
+        grn_obj_unlink(ctx, target);
+      }
     }
   }
 
