@@ -20,6 +20,7 @@
 #include "grn.h"
 #include "grn_db.h"
 #include "grn_arrow.h"
+#include "grn_output.h"
 
 #ifdef GRN_WITH_APACHE_ARROW
 #include <groonga/arrow.hpp>
@@ -656,14 +657,20 @@ namespace grnarrow {
       if (!weight_column) {
         return arrow::Status::OK();
       }
-      if (weight_column->type_id() != arrow::Type::INT32) {
+      float weight;
+      switch (weight_column->type_id()) {
+      case arrow::Type::INT32 :
+        weight = std::static_pointer_cast<arrow::Int32Array>(weight_column)->Value(index_);
+        break;
+      case arrow::Type::FLOAT :
+        weight = std::static_pointer_cast<arrow::FloatArray>(weight_column)->Value(index_);
+        break;
+      default :
         return arrow::Status::OK();
       }
 
       const auto &value =
         std::static_pointer_cast<arrow::StringArray>(value_column)->GetView(index_);
-      const auto &weight =
-        std::static_pointer_cast<arrow::Int32Array>(weight_column)->Value(index_);
       const grn_id domain = bulk_->header.domain;
       const auto raw_value = value.data();
       const auto raw_value_size = value.size();
@@ -690,18 +697,18 @@ namespace grnarrow {
         if (grn_type_id_is_text_family(ctx_, domain)) {
           const auto value_buffer_size =
             static_cast<unsigned int>(GRN_BULK_VSIZE(&value_buffer));
-          grn_vector_add_element(ctx_,
-                                 bulk_,
-                                 GRN_BULK_HEAD(&value_buffer),
-                                 value_buffer_size,
-                                 weight,
-                                 domain);
+          grn_vector_add_element_float(ctx_,
+                                       bulk_,
+                                       GRN_BULK_HEAD(&value_buffer),
+                                       value_buffer_size,
+                                       weight,
+                                       domain);
         } else {
           // TODO: Support weight vector for number such as Int64
-          grn_uvector_add_element(ctx_,
-                                  bulk_,
-                                  GRN_RECORD_VALUE(&value_buffer),
-                                  weight);
+          grn_uvector_add_element_record(ctx_,
+                                         bulk_,
+                                         GRN_RECORD_VALUE(&value_buffer),
+                                         weight);
         }
       }
       GRN_OBJ_FIN(ctx_, &value_buffer);
@@ -1434,7 +1441,7 @@ namespace grnarrow {
         std::min(nbytes,
                  static_cast<int64_t>(buffer_.size() - offset_));
       if (bytes_available > 0) {
-        memcpy(out, buffer_.data() + offset_, bytes_available);
+        grn_memcpy(out, buffer_.data() + offset_, bytes_available);
         offset_ += bytes_available;
         return bytes_available;
       } else {
@@ -1515,9 +1522,9 @@ namespace grnarrow {
                  "[arrow][stream-loader][consume] failed to resize buffer")) {
         return ctx_->rc;
       }
-      memcpy(buffer_->mutable_data() + current_buffer_size,
-             data,
-             data_size);
+      grn_memcpy(buffer_->mutable_data() + current_buffer_size,
+                 data,
+                 data_size);
 
       if (buffer_->size() < decoder_.next_required_size()) {
         return ctx_->rc;
@@ -1751,7 +1758,7 @@ namespace grnarrow {
 
     void close_record() {
       n_records_++;
-      if (n_records_ == 1024) {
+      if (n_records_ == grn_output_auto_flush_interval) {
         flush();
       }
     }
