@@ -40,6 +40,22 @@ extern "C" {
 
 #define GRN_N_RESERVED_TYPES 256
 
+/* #define GRN_REFERENCE_COUNT_DEBUG */
+#ifdef GRN_REFERENCE_COUNT_DEBUG
+# define grn_log_reference_count(...) printf(__VA_ARGS__)
+#else
+# define grn_log_reference_count(...)
+#endif
+
+#define GRN_TABLE_GROUPED (0x01<<0)
+#define GRN_TABLE_IS_GROUPED(table)\
+  ((table)->header.impl_flags & GRN_TABLE_GROUPED)
+#define GRN_TABLE_GROUPED_ON(table)\
+  ((table)->header.impl_flags |= GRN_TABLE_GROUPED)
+#define GRN_TABLE_IS_MULTI_KEYS_GROUPED(table)\
+  (GRN_TABLE_IS_GROUPED(table) &&\
+   table->header.domain == GRN_ID_NIL)
+
 extern bool grn_enable_reference_count;
 
 typedef struct _grn_db grn_db;
@@ -352,7 +368,7 @@ struct _grn_accessor {
   uint32_t reference_count;
 };
 
-enum {
+typedef enum {
   GRN_ACCESSOR_VOID = 0,
   GRN_ACCESSOR_GET_ID,
   GRN_ACCESSOR_GET_KEY,
@@ -367,7 +383,7 @@ enum {
   GRN_ACCESSOR_GET_DB_OBJ,
   GRN_ACCESSOR_LOOKUP,
   GRN_ACCESSOR_FUNCALL
-};
+} grn_accessor_action;
 
 #define DB_OBJ(obj) ((grn_db_obj *)obj)
 
@@ -377,25 +393,6 @@ void grn_obj_get_range_info(grn_ctx *ctx,
                             grn_id *range_id,
                             grn_obj_flags *range_flags);
 
-/* vector */
-
-/*
-typedef struct _grn_vector grn_vector;
-
-struct _grn_vector {
-  grn_obj str;
-  uint32_t *offsets;
-  int n_entries;
-};
-
-const char *grn_vector_fetch(grn_ctx *ctx, grn_obj *vector, int i, unsigned int *size);
-int grn_vector_delimit(grn_ctx *ctx, grn_obj *vector);
-int grn_vector_size(grn_ctx *ctx, grn_obj *vector);
-*/
-
-grn_rc grn_vector_delimit(grn_ctx *ctx, grn_obj *v, unsigned int weight, grn_id domain);
-grn_rc grn_vector_decode(grn_ctx *ctx, grn_obj *v, const char *data, uint32_t data_size);
-
 
 grn_rc grn_db_init_builtin_types(grn_ctx *ctx);
 
@@ -507,6 +504,11 @@ grn_obj *grn_obj_default_set_value_hook(grn_ctx *ctx,
                                         grn_obj **args,
                                         grn_user_data *user_data);
 
+grn_rc
+grn_column_get_all_token_columns(grn_ctx *ctx,
+                                 grn_obj *obj,
+                                 grn_obj *token_columns);
+
 grn_rc grn_pvector_fin(grn_ctx *ctx, grn_obj *obj);
 
 #ifdef __cplusplus
