@@ -94,6 +94,10 @@ const char *grn_proc_option_value_string(grn_ctx *ctx,
 grn_content_type grn_proc_option_value_content_type(grn_ctx *ctx,
                                                     grn_obj *option,
                                                     grn_content_type default_value);
+bool grn_proc_get_value_bool(grn_ctx *ctx,
+                             grn_obj *value,
+                             bool default_value,
+                             const char *context);
 int64_t grn_proc_get_value_int64(grn_ctx *ctx,
                                  grn_obj *value,
                                  int64_t default_value_raw,
@@ -102,6 +106,10 @@ grn_operator grn_proc_get_value_mode(grn_ctx *ctx,
                                      grn_obj *value,
                                      grn_operator default_mode,
                                      const char *context);
+grn_operator grn_proc_get_value_operator(grn_ctx *ctx,
+                                         grn_obj *value,
+                                         grn_operator default_operator,
+                                         const char *context);
 
 void grn_proc_output_object_name(grn_ctx *ctx, grn_obj *obj);
 void grn_proc_output_object_id_name(grn_ctx *ctx, grn_id id);
@@ -151,8 +159,7 @@ grn_rc grn_proc_syntax_expand_query(grn_ctx *ctx,
                                     const char *error_message_tag);
 
 grn_expr_flags grn_proc_expr_query_flags_parse(grn_ctx *ctx,
-                                               const char *query_flags,
-                                               size_t query_flags_size,
+                                               grn_obj *query_flags,
                                                const char *error_message_tag);
 
 grn_obj *grn_proc_lexicon_open(grn_ctx *ctx,
