@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2015 Brazil
-  Copyright(C) 2018-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2009-2015  Brazil
+  Copyright(C) 2018-2020  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -243,18 +243,18 @@ grn_index_column_build(grn_ctx *ctx, grn_obj *index_column)
     }
   }
   grn_obj_set_visibility(ctx, index_column, true);
-  for (int i = 0; i < n_columns; i++) {
-    if (!grn_obj_is_temporary(ctx, columns[i])) {
+  if (grn_enable_reference_count) {
+    for (int i = 0; i < n_columns; i++) {
       grn_obj_unlink(ctx, columns[i]);
     }
   }
   GRN_FREE(columns);
   grn_obj_touch(ctx, index_column, NULL);
 
-  if (target != src && !grn_obj_is_temporary(ctx, target)) {
+  if (target != src && grn_enable_reference_count) {
     grn_obj_unlink(ctx, target);
   }
-  if (!grn_obj_is_temporary(ctx, src)) {
+  if (grn_enable_reference_count) {
     grn_obj_unlink(ctx, src);
   }
 
@@ -1333,6 +1333,10 @@ grn_index_column_get_tokenizer(grn_ctx *ctx, grn_obj *index_column)
   }
 
   grn_table_get_info(ctx, lexicon, NULL, NULL, &tokenizer, NULL, NULL);
+  if (grn_enable_reference_count) {
+    grn_obj_unlink(ctx, lexicon);
+  }
+
   return tokenizer;
 }
 
@@ -1380,11 +1384,16 @@ grn_index_column_is_usable_range(grn_ctx *ctx,
   case GRN_TABLE_DAT_KEY :
     break;
   default :
+    if (grn_enable_reference_count) {
+      grn_obj_unlink(ctx, lexicon);
+    }
     return false;
-    break;
   }
 
   grn_table_get_info(ctx, lexicon, NULL, NULL, &tokenizer, NULL, NULL);
+  if (grn_enable_reference_count) {
+    grn_obj_unlink(ctx, lexicon);
+  }
   return tokenizer == NULL;
 }
 
@@ -1425,6 +1434,7 @@ grn_index_column_is_usable(grn_ctx *ctx,
   case GRN_OP_MATCH :
   case GRN_OP_NEAR :
   case GRN_OP_NEAR2 :
+  case GRN_OP_NEAR_PHRASE :
   case GRN_OP_SIMILAR :
   case GRN_OP_PREFIX :
   case GRN_OP_SUFFIX :
