@@ -178,7 +178,7 @@ grn_loader_get_column(grn_ctx *ctx,
   }
 
   id = grn_hash_add(ctx, loader->columns, name, name_length, &value, NULL);
-  memcpy(value, &column, sizeof(grn_obj *));
+  grn_memcpy(value, &column, sizeof(grn_obj *));
   grn_obj *range = grn_ctx_at(ctx, DB_OBJ(column)->range);
   if (grn_obj_is_table(ctx, range)) {
     GRN_PTR_PUT(ctx, &(loader->ranges), range);
@@ -257,7 +257,7 @@ add_weight_vector(grn_ctx *ctx,
   grn_obj weight_buffer;
 
   n = GRN_UINT32_VALUE(value);
-  GRN_UINT32_INIT(&weight_buffer, 0);
+  GRN_FLOAT32_INIT(&weight_buffer, 0);
   for (i = 0; i < n; i += 2) {
     grn_rc rc;
     grn_obj *key, *weight;
@@ -274,12 +274,12 @@ add_weight_vector(grn_ctx *ctx,
       grn_obj_unlink(ctx, range);
       break;
     }
-    grn_vector_add_element(ctx,
-                           vector,
-                           GRN_BULK_HEAD(key),
-                           GRN_BULK_VSIZE(key),
-                           GRN_UINT32_VALUE(&weight_buffer),
-                           key->header.domain);
+    grn_vector_add_element_float(ctx,
+                                 vector,
+                                 GRN_BULK_HEAD(key),
+                                 GRN_BULK_VSIZE(key),
+                                 GRN_FLOAT32_VALUE(&weight_buffer),
+                                 key->header.domain);
   }
   GRN_OBJ_FIN(ctx, &weight_buffer);
 }
@@ -1373,10 +1373,10 @@ grn_load_internal(grn_ctx *ctx, grn_load_input *input)
       ctx->impl->arrow_stream_loader =
         grn_arrow_stream_loader_open(ctx, &(ctx->impl->loader));
     }
-    grn_arrow_stream_loader_feed(ctx,
-                                 ctx->impl->arrow_stream_loader,
-                                 input->values.value,
-                                 input->values.length);
+    grn_arrow_stream_loader_consume(ctx,
+                                    ctx->impl->arrow_stream_loader,
+                                    input->values.value,
+                                    input->values.length);
 #else
     ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
         "[load][arrow] Apache Arrow support isn't enabled");
