@@ -20,10 +20,12 @@
 #include "grn.h"
 #include "grn_ctx_impl.h"
 #include "grn_dat.h"
+#include "grn_float.h"
 #include "grn_hash.h"
 #include "grn_ii.h"
 #include "grn_index_column.h"
 #include "grn_pat.h"
+#include "grn_store.h"
 
 grn_bool
 grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
@@ -44,11 +46,16 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
     case GRN_DB_UINT32 :
       return GRN_UINT32_VALUE(obj) != 0;
       break;
+    case GRN_DB_FLOAT32 : {
+      float float_value;
+      float_value = GRN_FLOAT32_VALUE(obj);
+      return grn_float32_is_zero(float_value);
+      break;
+    }
     case GRN_DB_FLOAT : {
       double float_value;
       float_value = GRN_FLOAT_VALUE(obj);
-      return (float_value < -DBL_EPSILON ||
-              DBL_EPSILON < float_value);
+      return grn_float_is_zero(float_value);
       break;
     }
     case GRN_DB_SHORT_TEXT :
@@ -133,16 +140,46 @@ grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_number_family(ctx, obj->header.domain);
 }
 
-grn_bool
+bool
 grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_VECTOR;
 }
 
+bool
+grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_vector(ctx, obj)) {
+    return false;
+  }
+
+  return obj->header.flags & GRN_OBJ_WITH_WEIGHT;
+}
+
+bool
+grn_obj_is_uvector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!obj) {
+    return false;
+  }
+
+  return obj->header.type == GRN_UVECTOR;
+}
+
+bool
+grn_obj_is_weight_uvector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_uvector(ctx, obj)) {
+    return false;
+  }
+
+  return obj->header.flags & GRN_OBJ_WITH_WEIGHT;
+}
+
 grn_bool
 grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 {
@@ -240,7 +277,7 @@ grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj)
   return (obj->header.flags & GRN_OBJ_WITH_WEIGHT) == GRN_OBJ_WITH_WEIGHT;
 }
 
-grn_bool
+bool
 grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
@@ -257,14 +294,17 @@ grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
     return GRN_FALSE;
   }
 
-  switch (range->header.type) {
+  uint8_t range_type = range->header.type;
+  grn_obj_unref(ctx, range);
+
+  switch (range_type) {
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
   case GRN_TABLE_DAT_KEY:
   case GRN_TABLE_NO_KEY:
-    return GRN_TRUE;
+    return true;
   default:
-    return GRN_FALSE;
+    return false;
   }
 }
 
@@ -614,8 +654,7 @@ grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj)
 }
 
 bool
-grn_obj_is_visible(grn_ctx *ctx,
-                   grn_obj *obj)
+grn_obj_is_visible(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
     return false;
@@ -665,28 +704,50 @@ grn_obj_set_visibility(grn_ctx *ctx,
     GRN_API_RETURN(ctx->rc);
   }
 
-  if (!grn_obj_is_index_column(ctx, obj)) {
+  if (!(grn_obj_is_index_column(ctx, obj) ||
+        grn_obj_is_token_column(ctx, obj))) {
     grn_obj inspected;
     GRN_TEXT_INIT(&inspected, 0);
     grn_inspect_limited(ctx, &inspected, obj);
     ERR(GRN_INVALID_ARGUMENT,
-        "[obj][set-visibility] must be index column: <%.*s>",
+        "[obj][set-visibility] "
+        "must be an index column or a token column: <%.*s>",
         (int)(GRN_TEXT_LEN(&inspected)),
         GRN_TEXT_VALUE(&inspected));
     GRN_OBJ_FIN(ctx, &inspected);
     GRN_API_RETURN(ctx->rc);
   }
 
-  grn_ii *ii = (grn_ii *)obj;
-  if (is_visible) {
-    ii->header.common->flags &= ~GRN_OBJ_INVISIBLE;
-  } else {
-    ii->header.common->flags |= GRN_OBJ_INVISIBLE;
+  if (grn_obj_is_index_column(ctx, obj)) {
+    grn_ii_set_visibility(ctx, (grn_ii *)obj, is_visible);
+  } else if (grn_obj_is_token_column(ctx, obj)) {
+    grn_ja_set_visibility(ctx, (grn_ja *)obj, is_visible);
   }
 
   GRN_API_RETURN(GRN_SUCCESS);
 }
 
+bool
+grn_obj_have_source(grn_ctx *ctx,
+                    grn_obj *obj)
+{
+  if (!obj) {
+    return false;
+  }
+
+  if (!GRN_DB_OBJP(obj)) {
+    return false;
+  }
+
+  return DB_OBJ(obj)->source_size > 0;
+}
+
+bool
+grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj)
+{
+  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj);
+}
+
 static void
 grn_db_reindex(grn_ctx *ctx, grn_obj *db)
 {
