@@ -38,6 +38,28 @@ class PackagesGroongaOrgPackageTask < PackageTask
     end
   end
 
+  def define_apt_task
+    namespace :apt do
+      desc "Clean APT repositories"
+      task :clean do
+        rm_rf("#{apt_dir}/repositories")
+      end
+    end
+    task apt: ["apt:clean"]
+    super
+  end
+
+  def define_yum_task
+    namespace :yum do
+      desc "Clean Yum repositories"
+      task :clean do
+        rm_rf("#{yum_dir}/repositories")
+      end
+    end
+    task yum: ["yum:clean"]
+    super
+  end
+
   def release(target_namespace)
     base_dir = __send__("#{target_namespace}_dir")
     repositories_dir = "#{base_dir}/repositories"
@@ -55,6 +77,7 @@ class PackagesGroongaOrgPackageTask < PackageTask
           release(target_namespace) if __send__("enable_#{target_namespace}?")
         end
       end
+      task target_namespace => ["#{target_namespace}:release"]
     end
   end
 
@@ -63,6 +86,6 @@ class PackagesGroongaOrgPackageTask < PackageTask
   end
 
   def dput_incoming
-    ENV["DPUT_INCOMING"] || "~groonga/ppa/ubuntu/"
+    ENV["DPUT_INCOMING"] || "~groonga/ubuntu/nightly"
   end
 end
